@@ -211,6 +211,7 @@ main (int argc, char **argv)
 
   program_name = argv[0];
   xmalloc_set_program_name (program_name);
+  bfd_set_error_program_name (program_name);
 
   expandargv (&argc, &argv);
 
@@ -1936,7 +1937,8 @@ powerpc_mangle_relocs (bfd *outbfd, asection *insec,
 
 	      if (rel->address > contents_size - 4)
 		{
-		  non_fatal (_("Out of range relocation: %lx"), rel->address);
+		  non_fatal (_("Out of range relocation: %lx"),
+			     (long) rel->address);
 		  break;
 		}
 	      
@@ -1991,7 +1993,8 @@ powerpc_mangle_relocs (bfd *outbfd, asection *insec,
 	    case 1:
 	      if (rel->address > contents_size - 2)
 		{
-		  non_fatal (_("Out of range relocation: %lx"), rel->address);
+		  non_fatal (_("Out of range relocation: %lx"),
+			     (long) rel->address);
 		  break;
 		}
 		       
@@ -2013,7 +2016,8 @@ powerpc_mangle_relocs (bfd *outbfd, asection *insec,
 	      /* PR 17512: file: 0455a112.  */
 	      if (rel->address > contents_size - 4)
 		{
-		  non_fatal (_("Out of range relocation: %lx"), rel->address);
+		  non_fatal (_("Out of range relocation: %lx"),
+			     (long) rel->address);
 		  break;
 		}
 		       
